/* GDA library
 * 
 * Copyright (C) Daniel Espinosa Ortiz 2008 <esodan@gmail.com>
 * 
 * This Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gda-repetitive-statement.h>
#include <libgda/gda-set.h>
#include <libgda/gda-holder.h>
#include <libgda/gda-connection.h>

typedef struct _GdaRepetitiveStatementPrivate GdaRepetitiveStatementPrivate;
struct _GdaRepetitiveStatementPrivate
{
	GdaStatement* statement;
	GSList* values_sets;
};

#define GDA_REPETITIVE_STATEMENT_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDA_TYPE_REPETITIVE_STATEMENT, GdaRepetitiveStatementPrivate))

enum {
	PROP_0,
	PROP_STATEMENT
};



G_DEFINE_TYPE (GdaRepetitiveStatement, gda_repetitive_statement, G_TYPE_OBJECT);

static void
gda_repetitive_statement_init (GdaRepetitiveStatement *object)
{
	GdaRepetitiveStatementPrivate *priv;
	
	priv = GDA_REPETITIVE_STATEMENT_PRIVATE (object);
	
	priv->statement = NULL;
	priv->values_sets = NULL;
}

static void unref_object (gpointer data, gpointer user_data)
{
	g_object_unref (data);
}

static void
gda_repetitive_statement_finalize (GObject *object)
{
	GdaRepetitiveStatementPrivate *priv;
	
	priv = GDA_REPETITIVE_STATEMENT_PRIVATE (object);
	
	g_object_unref (priv->statement);
	g_slist_foreach (priv->values_sets, unref_object, NULL);
	g_slist_free (priv->values_sets);

	G_OBJECT_CLASS (gda_repetitive_statement_parent_class)->finalize (object);
}

static void
gda_repetitive_statement_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
{
	GdaRepetitiveStatementPrivate *priv;
	
	g_return_if_fail (GDA_IS_REPETITIVE_STATEMENT (object));
	
	priv = GDA_REPETITIVE_STATEMENT_PRIVATE (object);
	switch (prop_id) {
	case PROP_STATEMENT:
		priv->statement = g_value_dup_object (value);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
gda_repetitive_statement_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
{
	GdaRepetitiveStatementPrivate *priv;
	
	g_return_if_fail (GDA_IS_REPETITIVE_STATEMENT (object));

	priv = GDA_REPETITIVE_STATEMENT_PRIVATE (object);
	switch (prop_id) {
	case PROP_STATEMENT:
		g_value_set_object (value, priv->statement);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
gda_repetitive_statement_class_init (GdaRepetitiveStatementClass *klass)
{
	GObjectClass* object_class = G_OBJECT_CLASS (klass);
	GObjectClass* parent_class = G_OBJECT_CLASS (klass);

	g_type_class_add_private (klass, sizeof (GdaRepetitiveStatementPrivate));

	object_class->finalize = gda_repetitive_statement_finalize;
	object_class->set_property = gda_repetitive_statement_set_property;
	object_class->get_property = gda_repetitive_statement_get_property;

	g_object_class_install_property (object_class,
	                                 PROP_STATEMENT,
	                                 g_param_spec_object ("statement",
	                                                      "stm",
	                                                      "Statement to Execute",
	                                                      GDA_TYPE_STATEMENT,
	                                                      G_PARAM_WRITABLE | G_PARAM_READABLE | G_PARAM_CONSTRUCT_ONLY));
}

/**
 * gda_repetitive_statement_new
 * @template: a #GdaStatement object
 *
 * Creates a new #GdaRepetitiveStatement object using @stmt as a template.
 *
 * Returns: a new #GdaRepetitiveStatement object
 */
GdaRepetitiveStatement*
gda_repetitive_statement_new (GdaStatement *template)
{
	GdaRepetitiveStatement *rstmt;
	
	rstmt = GDA_REPETITIVE_STATEMENT (g_object_new (GDA_TYPE_REPETITIVE_STATEMENT, "statement", template, NULL));
	
	return rstmt;
}

/**
 * gda_repetitive_statement_get_template_set
 * @rstmt: a #GdaRepetitiveStatement object
 * @set: a place to store the returned template set
 * @error: a place to store error, or %NULL
 *
 * Gets a new #GdaSet object with the parameters used by the template statement in the
 * @rstmt object. 
 *
 * Use this object with gda_repetitive_statement_append_values_set().
 *
 * Returns: TRUE on success, FALSE on error
 */
gboolean
gda_repetitive_statement_get_template_set (GdaRepetitiveStatement *rstmt, GdaSet **set, GError **error)
{
	GdaRepetitiveStatementPrivate *priv;
	
	priv = GDA_REPETITIVE_STATEMENT_PRIVATE (rstmt);
	
	return gda_statement_get_parameters (priv->statement, set, error);
}

/**
 * gda_repetitive_statement_add_values_set
 * @rstmt: a #GdaRepetitiveStatement object
 * @values: a #GdaSet object with the values to be used
 * @error: a place to store errors, or %NULL
 *
 * Specifies that @rstmt be executed one time with the values contained in @values. This method 'steals' the
 * reference to @values.
 *
 * A new #GdaSet to be used as the @values argument can be obtained using
 * gda_repetitive_statement_get_template_set().
 *
 * Returns: a new #GdaRepetitiveStatement object
 */
gboolean
gda_repetitive_statement_append_values_set (GdaRepetitiveStatement *rstmt, GdaSet *values)
{
	GdaRepetitiveStatementPrivate *priv;
	
	g_return_val_if_fail (GDA_IS_REPETITIVE_STATEMENT(rstmt), FALSE);
	g_return_val_if_fail (GDA_IS_SET (values), FALSE);
	
	priv = GDA_REPETITIVE_STATEMENT_PRIVATE (rstmt);
	
	priv->values_sets = g_slist_prepend (priv->values_sets, values);
	
	return TRUE;
}

/**
 * gda_repetitive_statement_get_all_values_sets
 * @rstmt: a #GdaRepetitiveStatement object
 *
 * Get all the values sets which will have been added using gda_repetitive_statement_add_values_set().
 *
 * Returns: a #GSList of #GdaSet objects
 */
const GSList*
gda_repetitive_statement_get_all_values_sets (GdaRepetitiveStatement *rstmt)
{
	GdaRepetitiveStatementPrivate *priv;
	
	priv = GDA_REPETITIVE_STATEMENT_PRIVATE (rstmt);
	
	return priv->values_sets;
}
